<?php

namespace Drupal\eca_base\Event;

use Cron\CronExpression;
use Drupal\Component\EventDispatcher\Event;
use Drupal\eca\EcaState;
use Drupal\eca\Event\ConditionalApplianceInterface;

/**
 * Provides a cron event.
 *
 * @internal
 *   This class is not meant to be used as a public API. It is subject for name
 *   change or may be removed completely, also on minor version updates.
 *
 * @package Drupal\eca_base\Event
 */
class CronEvent extends Event implements ConditionalApplianceInterface {

  /**
   * ECA state service.
   *
   * @var \Drupal\eca\EcaState
   */
  protected EcaState $state;

  /**
   * Constructs a new CronEvent object.
   *
   * @param \Drupal\eca\EcaState $state
   *   The ECA state service.
   */
  public function __construct(EcaState $state) {
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   *
   * Verifies if this event is due for the next execution.
   *
   * This event stores the last execution time for each modeller event
   * identified by $id and determines with the given frequency, if and when
   * this same event triggered cron should be executed.
   */
  public function appliesForLazyLoadingWildcard(string $wildcard): bool {
    [$id, $frequency] = explode('::', $wildcard, 2);
    return $this->isDue($id, $frequency);
  }

  /**
   * {@inheritdoc}
   *
   * When this event is due for next execution, this also stores the current
   * time as the new "last executed" timestamp in the ECA state.
   */
  public function applies(string $id, array $arguments): bool {
    if ($this->isDue($id, $arguments['frequency'])) {
      $this->storeTimestamp($id);
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Determines, if the cron with $id is due for next execution.
   *
   * It receives the last execution time of this event cron and calculates
   * by the given frequency, if the next execution time has already been
   * passed and returns TRUE, if so.
   *
   * @param string $id
   *   The id of the modeller event.
   * @param string $frequency
   *   The frequency as a cron pattern.
   *
   * @return bool
   *   TRUE, if the event $id is due for next execution, FALSE otherwise.
   */
  protected function isDue(string $id, string $frequency): bool {
    $currentTime = $this->state->getCurrentTimestamp();
    $lastRun = $this->state->getTimestamp('cron-' . $id);

    // Cron's maximum granularity is on minute level. Therefore we round the
    // current time to the last passed minute. That way we avoid accidental
    // concurrent runs.
    $currentTime -= ($currentTime % 60);
    $lastRun -= ($lastRun % 60);

    try {
      return $currentTime >= $this->getNextRunTimestamp($lastRun, $frequency);
    }
    catch (\Exception $e) {
      // @todo Log this exception.
    }
    return FALSE;
  }

  /**
   * Calculates the timestamp for the next execution.
   *
   * @param int $lastRunTimestamp
   *   Timestamp, when it was executed last or 0 if it never ran before.
   * @param string $frequency
   *   The frequency as a cron pattern.
   *
   * @return int
   *   Timestamp for next execution.
   *
   * @throws \Exception
   */
  public function getNextRunTimestamp(int $lastRunTimestamp, string $frequency): int {
    $cron = new CronExpression($frequency);
    $dt = new \DateTime();
    $dt
      ->setTimezone(new \DateTimeZone('UTC'))
      ->setTimestamp($lastRunTimestamp);
    return $cron->getNextRunDate($dt)->getTimestamp();
  }

  /**
   * Stores the execution time for the modeller event $id in ECA state.
   *
   * @param string $id
   *   The id of the modeller event.
   * @param int|null $timestamp
   *   (optional) The timestamp value to store. When not given, the current time
   *   will be used.
   */
  protected function storeTimestamp(string $id, ?int $timestamp = NULL): void {
    $this->state->setTimestamp('cron-' . $id, $timestamp);
  }

}
